@@ -3,18 +3,22 @@ from json.decoder import JSONDecodeError
 import requests
 from requests.exceptions import ConnectionError
 import urllib3
-from dataclasses import dataclass
-from absbox.local.util import mkTag,query,isDate,flat
+from dataclasses import dataclass,field
+from absbox.local.util import mkTag,isDate,flat
 from absbox.local.component import mkPool,mkAssumption,mkAssumption2
+from absbox.local.base import *
 import pandas as pd
+from pyspecter import S,query
 
 urllib3.disable_warnings()
 
 @dataclass
 class API:
-    url: str
+    url:str
+    #lang = field(repr=False, default="chinese")
+    lang:str = "chinese"
     server_info = {}
-    version = "0","6","1"
+    version:tuple = "0","8","1"
     hdrs = {'Content-type': 'application/json', 'Accept': 'text/plain'}
 
     def __post_init__(self):
@@ -28,7 +32,7 @@ class API:
         echo = json.loads(_r)
         self.server_info = echo
         x,y,z = echo['version'].split(".")
-        logging.info(f"Connect Successfully with engine version {echo['version']}")
+        logging.info(f"Connect with engine version {echo['version']} successfully")
         if self.version[1] != y:
             logging.error(f"Failed to init the api instance, lib support={self.version} but server version={echo['version']} , pls upgrade your api package by: pip -U absbox")
             return
@@ -52,31 +56,32 @@ class API:
                           ,ensure_ascii=False)
 
     def _validate_assump(self,x,e,w):
+        def asset_check(_e,_w):
+            return _e,_w
         a = x['assump']
         asset_ids = set(range(len(query(x,['deal','contents','pool','assets']))))
         match a:
-            case {'tag':'Single',
-                'contents':{'tag':'ByIndex',
-                            'contents':(assumps,_)}}:
+            case {'tag':'Single','contents':{'tag':'PoolLevel'}}:
+                return [True,e,w]
+            case {'tag':'Multiple','contents':{'tag':'PoolLevel'}}:
+                return [True,e,w]
+            case {'tag':'Single', 'contents':{'tag':'ByIndex', 'contents':(assumps,_)}}:
                 _ids = set(flat([ assump[0] for assump in assumps ]))
                 if not _ids.issubset(asset_ids):
                     e.append(f"Not Valid Asset ID:{_ids - asset_ids}")
                 missing_asset_id = asset_ids - _ids
                 if len(missing_asset_id)>0:
                     w.append(f"Missing Asset to set assumption:{missing_asset_id}")            
-            case {'tag':'Multiple',
-                'contents':scenarioMap}:
+            case {'tag':'Multiple', 'contents':scenarioMap}:
                 for k,v in scenarioMap.items():
+                    if v['tag']=='PoolLevel':
+                        continue
                     _ids = set(flat([ _a[0] for _a in v['contents'][0]]))
                     if not _ids.issubset(asset_ids):
                         e.append(f"Scenario:{k},Not Valid Asset ID:{_ids - asset_ids}")
                     missing_asset_id = asset_ids - _ids
                     if len(missing_asset_id)>0:
                         w.append(f"Scenario:{k},Missing Asset to set assumption:{missing_asset_id}")
-            case {'tag':'Single','contents':{'tag':'PoolLevel'}}:
-                return [True,e,w]
-            case {'tag':'Multiple','contents':{'tag':'PoolLevel'}}:
-                return [True,e,w]
             case None:
                 return [True,e,w]
             case _ :
@@ -186,7 +191,7 @@ class API:
                 c = _f.read()
                 deal = pickle.loads(c)
 
-        # construst request
+        # construct request
         req = self.build_req(deal, assumptions, pricing)
 
         #validate deal
@@ -211,10 +216,13 @@ class API:
         result = self._send_req(req,url)
 
         if read:
-            result = pd.DataFrame([_['contents'] for _ in result]
-                                                  , columns=["日期", "未偿余额", "本金", "利息", "早偿金额", "违约金额", "回收金额", "损失", "利率"])
-            result = result.set_index("日期")
-            result.index.rename("日期", inplace=True)
+            flow_header,idx = guess_pool_flow_header(result[0],self.lang)
+            try:
+                result = pd.DataFrame([_['contents'] for _ in result] , columns=flow_header)
+            except ValueError as e:
+                logging.error(f"Failed to match header:{flow_header} with {result[0]['contents']}")
+            result = result.set_index(idx)
+            result.index.rename(idx, inplace=True)
             result.sort_index(inplace=True)
         return result
     
@@ -233,6 +241,23 @@ class API:
         except JSONDecodeError as e:
             raise RuntimeError(e)        
 
+def guess_pool_flow_header(x,l):
+    match (x['tag'],l):
+        case ('MortgageFlow','chinese'):
+            return (china_mortgage_flow_fields_d,"日期")
+        case ('MortgageFlow','english'):
+            return (english_mortgage_flow_fields_d,"Date")
+        case ('LoanFlow','chinese'):
+            return (china_loan_flow_d,"日期")
+        case ('LoanFlow','english'):
+            return (english_loan_flow_d,"Date")
+        case ('LeaseFlow','chinese'):
+            return (china_rental_flow_d,"日期")
+        case ('LeaseFlow','english'):
+            return (english_rental_flow_d,"Date")
+        case _:
+            raise RuntimeError(f"Failed to match pool header with {x['tag']}{l}")
+
 
 def save(deal,p:str):
     def save_to(b):
