@@ -1,7 +1,9 @@
-from absbox.local.util import mkTag,DC,mkTs,query
+from absbox.local.util import mkTag,DC,mkTs
 from enum import Enum
 import itertools
 
+from pyspecter import query
+
 datePattern = {"月末":"MonthEnd"
               ,"季度末":"QuarterEnd"
               ,"年末":"YearEnd"
@@ -21,6 +23,10 @@ freqMap = {"每月": "Monthly"
     , "Quarterly": "Quarterly"
     , "SemiAnnually": "SemiAnnually"
     , "Annually": "Annually"
+    , "monthly": "Monthly"
+    , "quarterly": "Quarterly"
+    , "semiAnnually": "SemiAnnually"
+    , "annually": "Annually"
     }
 
 baseMap = {"资产池余额": "CurrentPoolBalance"
@@ -578,7 +584,7 @@ def mkTriggerEffect(x):
     match x:
         case ("新状态",s) | ("newStatus",s):
             return mkTag(("DealStatusTo", mkStatus(s)))
-        case ["计提费用",*fn] | ["accureFees",*fn]:
+        case ["计提费用",*fn] | ["accrueFees",*fn]:
             return mkTag(("DoAccrueFee", fn))
         case ["新增事件",trg] | ["newTrigger",trg]:
             return mkTag(("AddTrigger", mkTrigger(trg)))
@@ -624,24 +630,26 @@ def mkWaterfall(r, x):
 
 def mkAssetRate(x):
     match x:
-        case ["固定",r] | ["Fixed",r]:
+        case ["固定",r] | ["fix",r]:
             return mkTag(("Fix",r))
         case ["浮动",r,{"基准":idx,"利差":spd,"重置频率":p}]:
             return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
         case ["floater",r,{"index":idx,"spread":spd,"reset":p}]:
             return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
+        case ["Floater",r,{"index":idx,"spread":spd,"reset":p}]:
+            return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkAssetRate")
 
 def mkAmortPlan(x)->dict:
     match x:
-        case "等额本息" | "Level" :
+        case "等额本息" | "Level" | "level":
             return mkTag("Level")
-        case "等额本金" | "Even" :
+        case "等额本金" | "Even" | "even":
             return mkTag("Even")
-        case "先息后本" | "I_P" :
+        case "先息后本" | "I_P" | "i_p" :
             return mkTag("I_P")
-        case "等本等费" | "F_P" :
+        case "等本等费" | "F_P" | "f_p" :
             return mkTag("F_P")
         case _ :
             raise RuntimeError(f"Failed to match AmortPlan {x}:mkAmortPlan")
@@ -650,6 +658,8 @@ def mkAmortPlan(x)->dict:
 def mkAsset(x):
     _statusMapping = {"正常": mkTag(("Current"))
                     , "违约": mkTag(("Defaulted",None))
+                    , "current":mkTag(("Current"))
+                    , "defaulted": mkTag(("Defaulted",None))
                     , "Current":mkTag(("Current"))
                     , "Defaulted": mkTag(("Defaulted",None))
                     }
@@ -666,6 +676,10 @@ def mkAsset(x):
              ,"currentRate": currentRate
              ,"remainTerm": remainTerms
              ,"status": status}]:
+
+            borrowerNum1 = x[2].get("borrowerNum",None)
+            borrowerNum2 = x[2].get("借款数量",None)
+
             return mkTag(("Mortgage",[
                                       {"originBalance": originBalance,
                                       "originRate": mkAssetRate(originRate),
@@ -677,6 +691,7 @@ def mkAsset(x):
                                      currentBalance,
                                      currentRate,
                                      remainTerms,
+                                     (borrowerNum1 or borrowerNum2),
                                      _statusMapping[status]]))
         case ["贷款"
             ,{"放款金额": originBalance, "放款利率": originRate, "初始期限": originTerm
@@ -685,7 +700,7 @@ def mkAsset(x):
              ,"当前利率": currentRate
              ,"剩余期限": remainTerms
              ,"状态": status}] \
-            |["loan"
+            |["Loan"
             ,{"originBalance": originBalance, "originRate": originRate, "originTerm": originTerm
                   ,"freq": freq, "type": _type, "originDate": startDate}
             ,{"currentBalance": currentBalance
@@ -707,11 +722,11 @@ def mkAsset(x):
         case ["分期"
             ,{"放款金额": originBalance, "放款费率": originRate, "初始期限": originTerm
                   ,"频率": freq, "类型": _type, "放款日": startDate}
-            ,{"状态": status}] \
-            |["installment"
+            ,{"当前余额":currentBalance, "状态": status}] \
+            |["Installment"
             ,{"originBalance": originBalance, "feeRate": originRate, "originTerm": originTerm
                   ,"freq": freq, "type": _type, "originDate": startDate}
-            ,{"status": status}]:
+            ,{"currentBalance":currentBalance,"status": status}]:
             return mkTag(("Installment",[
                                       {"originBalance": originBalance,
                                       "originRate": mkAssetRate(originRate),
@@ -720,9 +735,39 @@ def mkAsset(x):
                                       "startDate": startDate,
                                       "prinType": mkAmortPlan(_type)
                                       } | mkTag("LoanOriginalInfo"),
-                                     0.0,
+                                     currentBalance,
                                      _statusMapping[status]]))       
-
+        case ["租赁"
+                ,{"固定租金": dailyRate, "初始期限": originTerm
+                  ,"频率": dp, "起始日": startDate,"状态":status,"剩余期限":remainTerms}] \
+            |["Lease"
+                ,{"fixRental": dailyRate, "originTerm": originTerm
+                  ,"freq": dp, "originDate": startDate, "status":status,"remainTerm":remainTerms}]:
+            return mkTag(("RegularLease"
+                            ,[{"originTerm": originTerm, "startDate": startDate, "paymentDates": mkDatePattern(dp),"originRental":dailyRate} | mkTag("LeaseInfo")
+                              ,0
+                              ,remainTerms
+                              ,_statusMapping[status]]))       
+        case ["租赁"
+                ,{"初始租金": dailyRate, "初始期限": originTerm
+                  ,"频率": dp, "起始日": startDate,"计提周期":accDp,"涨幅":rate,"状态":status,"剩余期限":remainTerms}] \
+            |["Lease"
+                ,{"initRental": dailyRate, "originTerm": originTerm
+                  ,"freq": dp, "originDate": startDate,"accrue":accDp
+                  ,"pct":rate,"status":status,"remainTerm":remainTerms}]:
+            
+            dailyRatePlan = None
+            _stepUpType = "curve" if isinstance(rate, list) else "constant"
+            if _stepUpType == "constant":
+                dailyRatePlan = mkTag(("FlatRate",[mkDatePattern(accDp),rate]))
+            else:
+                dailyRatePlan = mkTag(("ByRateCurve",[mkDatePattern(accDp),rate]))
+            return mkTag(("StepUpLease"
+                            ,[ {"originTerm": originTerm, "startDate": startDate, "paymentDates": mkDatePattern(dp),"originRental":dailyRate} | mkTag("LeaseInfo")
+                              ,dailyRatePlan
+                              ,0
+                              ,remainTerms
+                              ,_statusMapping[status]]))       
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkAsset")
 
@@ -733,12 +778,16 @@ def identify_deal_type(x):
             return "LDeal"
         case {"pool":{"assets":[{'tag':'Mortgage'},*rest]}} :
             return "MDeal"
+        case {"pool":{"assets":[],"futureCf":cfs}} if cfs[0]['tag']=='MortgageFlow' :
+            return "MDeal"
         case {"pool":{"assets":[{'tag':'Installment'},*rest]}} :
             return "IDeal"
+        case {"pool":{"assets":[{'tag':'Lease'},*rest]}} | {"pool":{"assets":[{'tag':'RegularLease'},*rest]}}:
+            return "RDeal"
+        case {"pool":{"assets":[{'tag':'StepUpLease'},*rest]}}:
+            return "RDeal"
         case _ :
-            return "MDeal"
-        #case _ :
-        #    raise RuntimeError(f"Failed to identify deal type {x}")
+            raise RuntimeError(f"Failed to identify deal type {x}")
 
 
 def mkCallOptions(x):
@@ -754,9 +803,9 @@ def mkCallOptions(x):
         case {"指定日之后": d} | {"afterDate": d}:
             return mkTag(("AfterDate", d))
         case {"任意满足": xs} | {"or": xs}:
-            return mkTag(("Or", xs))
+            return mkTag(("Or", [ mkCallOptions(_x) for _x in xs ]))
         case {"全部满足": xs} | {"and": xs}:
-            return mkTag(("And", xs))
+            return mkTag(("And", [ mkCallOptions(_x) for _x in xs ]))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkCallOptions")
             
@@ -787,6 +836,16 @@ def mkAssumption(x) -> dict:
             return mkTag(("CallWhen",[mkCallOptions(co) for co in opts]))
         case {"停止": d} | {"StopAt":d}:
             return mkTag(("StopRunBy",d))
+        case {"租赁截止日": d} | {"LeaseProjectEnd":d}:
+            return mkTag(("LeaseProjectionEnd",d))
+        case {"租赁年涨幅": r} | {"LeaseAnnualIncreaseRate":r} if not isinstance(r, list):
+            return mkTag(("LeaseBaseAnnualRate",r))
+        case {"租赁年涨幅": r} | {"LeaseAnnualIncreaseRate":r}:
+            return mkTag(("LeaseBaseCurve",mkTs("FloatCurve",r)))
+        case {"租赁间隔": n} | {"LeaseGapDays":n}:
+            return mkTag(("LeaseGapDays",n))
+        case {"租赁间隔表": (tbl,n)} | {"LeaseGapDaysByAmount":(tbl,n)}:
+            return mkTag(("LeaseGapDaysByAmount",[tbl,n]))
         case _ :
             raise RuntimeError(f"Failed to match {x}:Assumption")
 
@@ -805,7 +864,7 @@ def mkAssumption2(x) -> dict:
             raise RuntimeError(f"Failed to match {x}:mkAssumption2")
 
 def mkPool(x):
-    mapping = {"LDeal":"LPool","MDeal":"MPool","IDeal":"IPool"}
+    mapping = {"LDeal":"LPool","MDeal":"MPool","IDeal":"IPool","RDeal":"RPool"}
     match x:
         case {"清单":assets,"封包日":d} | {"assets":assets,"cutoffDate":d}:
             _pool = {"assets": [mkAsset(a) for a in assets],"asOfDate": d}
@@ -844,17 +903,19 @@ def mkCf(x):
     if len(x)==0:
         return None
     else:
-        return [ mkTag(("MortgageFlow",_x+([0.0]*5))) for _x in x]
+        return [ mkTag(("MortgageFlow",_x+[0.0]*5+[None])) for _x in x]
 
 def mkCollection(xs):
     sourceMapping = {"利息回款": "CollectedInterest"
                     , "本金回款": "CollectedPrincipal"
                     , "早偿回款": "CollectedPrepayment"
                     , "回收回款": "CollectedRecoveries"
+                    , "租金回款": "CollectedRental"
                     ,"CollectedInterest": "CollectedInterest"
                     , "CollectedPrincipal": "CollectedPrincipal"
                     , "CollectedPrepayment": "CollectedPrepayment"
                     , "CollectedRecoveries": "CollectedRecoveries"
+                    , "CollectedRental": "CollectedRental"
                     }
     return [[sourceMapping[x], acc] for (x, acc) in xs]
 
