@@ -2,6 +2,7 @@ from dataclasses import dataclass
 import functools
 from absbox.local.util import mkTag
 from absbox.local.component import *
+from absbox.local.base import * 
 import pandas as pd
 import collections
 
@@ -35,7 +36,7 @@ class Generic:
             "dates": parsedDates,
             "name": self.name,
             "status":mkTag((self.status)),
-            "pool": {"assets": [mkAsset(x) for x in self.pool.get('breakdown', [])]
+            "pool": {"assets": [mkAsset(x) for x in self.pool.get('assets', [])]
                      , "asOfDate": self.dates['cutoff']
                      , "issuanceStat": self.pool.get("issuanceStat")
                      , "futureCf":mkCf(self.pool.get('cashflow', [])) },
@@ -85,15 +86,9 @@ class Generic:
         return None
 
     def read(self, resp, position=None):
-        read_paths = {'bonds': ('bndStmt'
-                               , ["date", "balance", "interest", "principal", "rate", "cash", "memo"]
-                               , "bond")
-                     , 'fees': ('feeStmt'
-                               , ["date", "balance", "payment", "due", "memo"]
-                                , "fee")
-                     , 'accounts': ('accStmt'
-                                 , ["date", "balance", "change", "memo"]
-                                 , "account")}
+        read_paths = {'bonds': ('bndStmt' , english_bondflow_fields , "bond")
+                     , 'fees': ('feeStmt' , english_fee_flow_fields_d , "fee")
+                     , 'accounts': ('accStmt' , english_acc_flow_fields_d , "account")}
         output = {}
         for comp_name, comp_v in read_paths.items():
             output[comp_name] = {}
@@ -126,9 +121,10 @@ class Generic:
 
         output['pool'] = {}
         output['pool']['flow'] = pd.DataFrame([_['contents'] for _ in resp[0]['pool']['futureCf']]
-                                              , columns=["date", "balance", "principal", "interest", "prepayment", "default", "recovery", "loss", "rate"])
-        output['pool']['flow'] = output['pool']['flow'].set_index("date")
-        output['pool']['flow'].index.rename("date", inplace=True)
+                                              , columns=english_mortgage_flow_fields_d)
+        pool_idx = 'Date'
+        output['pool']['flow'] = output['pool']['flow'].set_index(pool_idx)
+        output['pool']['flow'].index.rename(pool_idx, inplace=True)
 
         output['pricing'] = pd.DataFrame.from_dict(resp[3]
                                                    , orient='index'
